import data, os, sys, pygame
from pygame.locals import *
import globals
from globals import *
from euclid import Vector2


class GameObject(pygame.sprite.Sprite):

  def __init__(self, x=100, y=100):
    pygame.sprite.Sprite.__init__(self) 
    self.rect = self.image.get_rect()
    self.rect_place((x,y))

  def rect_move(self, xy):
    self.rect.move_ip(xy)

  def rect_place(self, xy):
    self.rect.center = xy


  def rect_bg_collide(self, side):
    if side == 'top':
      x, y = self.rect.midtop
    elif side == 'left':
      x, y = self.rect.midleft
    elif side == 'right':
      x, y = self.rect.midright
    elif side == 'bottom':
      x, y = self.rect.midbottom
    return self.bg_collide(x, y)


  def bg_collide(self, x, y):
    if x >= globals.BGIR.right  or x < globals.BGIR.left or \
       y >= globals.BGIR.bottom or y < globals.BGIR.top:
      return True
    else:
      return globals.BGIA[ x+XOFF, y+YOFF ]



class Character(GameObject):
  maxup = 12
  maxdown = 16
  maxleft = 5
  maxright = 5


  def __init__(self, x=100, y=100):
    pygame.sprite.Sprite.__init__(self) 
    self.image = self.frame_images['default'][0]
    self.rect = self.image.get_rect().copy()
    self.colrect = self.rect.inflate(-24, -12)
    self.visrect = self.rect.inflate(300, 800)
    self.rect_place((x,y))
    self.frame_num = {}
    self.m = 50
    self.v = Vector2(0, 0)
    self.f = Vector2(0, 0)
    self.a = Vector2(0, 0)
    self.speed = Vector2(-self.maxleft, 0)
    self.dir = K_LEFT
    for action in self.frame_images:
      self.frame_num[action] = 0


  @staticmethod
  def init_frames(frames_list, color_key=None):
    frame_images = {}
    flipped_frame_images = {}
    frame_step = {}
    for action in frames_list:
      frame_images[action] = []
      flipped_frame_images[action] = []
      finfo = frames_list[action]
      for img in finfo['images']:
        img = data.load_image(img, color_key)[0]
        frame_images[action].append(img)
        flipped_frame_images[action].append( pygame.transform.flip(img, True, False) )

      if finfo.has_key('anim_time'):
        anim_time = finfo['anim_time']
      else:
        anim_time = float(len(frame_images[action])*3) / FPS
      frame_step[action] = \
        1.0 / (FPS * anim_time / len(frame_images[action]))
    return (frame_images, flipped_frame_images, frame_step)


  def frame_update(self, state):
    fnum = self.frame_num[state]
    if self.dir == K_RIGHT:
      frames = self.flipped_frame_images[state]
    else:
      frames = self.frame_images[state]
    self.image = frames[ int(fnum) ]

    fnum += self.frame_step[state]
    if fnum >= len(frames):
      fnum = 0
    self.frame_num[state] = fnum


  def colrect_bg_collide(self, side, offset=0, offset2=0):
    colrect = self.colrect
    if side == 'top':
      x, y = colrect.midtop
      x2, y2 = colrect.topleft
      x3, y3 = colrect.topright
      y -= offset; y2 -= offset; y3 -= offset
      x += offset2; x2 += offset2; x3 += offset2
    elif side == 'left':
      x, y = colrect.midleft
      x2, y2 = colrect.topleft
      x3, y3 = colrect.bottomleft
      x -= offset; x2 -= offset; x3 -= offset
      y += offset2; y2 += offset2; y3 += offset2
    elif side == 'right':
      x, y = colrect.midright
      x2, y2 = colrect.topright
      x3, y3 = colrect.bottomright
      x += offset; x2 += offset; x3 += offset
      y += offset2; y2 += offset2; y3 += offset2
    elif side == 'bottom':
      x, y = colrect.midbottom
      x2, y2 = colrect.bottomleft
      x3, y3 = colrect.bottomright
      y += offset; y2 += offset; y3 += offset
      x += offset2; x2 += offset2; x3 += offset2
    return self.bg_collide(x, y) or \
           self.bg_collide(x2, y2) or \
           self.bg_collide(x3, y3)


  def gravity(self):
    #self.f += DIR.down * self.m * 1.3 # g=1.3
    self.v += GRAVITY
  

  def phy_step(self):
    self.gravity()
    if self.f.y != 0:
      self.a = self.f / self.m
      self.v += self.a
      self.f.y = 0
    #if self.__class__.__name__ == 'Player': print self.f
    #self.f.x = 0; self.f.y = 0


  def move(self):
    vx = int(self.v.x + self.speed.x)
    self.movex(vx)

    self.v.y = min(self.v.y,  self.maxdown)
    self.v.y = max(self.v.y, -self.maxup)
    vy = int(self.v.y + self.speed.y)
    self.movey(vy)


  def movey(self, vy):
    vy = int(vy)
    if vy > 0:
      side = 'bottom'
      offset = 1
    elif vy < 0:
      side = 'top'
      offset = -1
    else: 
      return
    avy = abs(vy)

    if avy <= 8:
      if not self.colrect_bg_collide(side, avy):
        self.rect_move((0, vy))
      #elif not self.colrect_bg_collide(side, avy/2):
        #self.v.y = 0
        #self.rect_move((0, vy/2))
        #self.movey(vy/2 - offset)
      else:
        self.v.y = 0
        self.movey(vy/2.0)
    else:
      if not self.colrect_bg_collide(side, 8):
        self.rect_move((0, offset*8))
        self.movey(vy - offset*8)
      else:
        self.v.y = 0
        self.movey(offset*7)


  def movex(self, vx):
    vx = int(vx)
    vel = Vector2(0,0)
    if vx > 0:
      side = 'right'
      offset = 1
    elif vx < 0:
      side = 'left'
      offset = -1
    else:
      return
    avx = abs(vx)

    if self.colrect_bg_collide('bottom', 5):
      # TODO use above check only for the player, and nobody else - perf
      if not self.colrect_bg_collide(side, avx, avx):
        vel.x = vx
        vel.y = avx
      elif not self.colrect_bg_collide(side, avx):
        self.rect_move((vx, 0))
        return
      elif not self.colrect_bg_collide(side, avx, -avx):
        vel.x = vx
        vel.y = -avx
      elif not self.colrect_bg_collide(side, avx/2, -avx):
        vel.x = vx/2
        vel.y = -avx
      #elif not self.colrect_bg_collide(side, avx/2):
        #self.rect_move((vx/2, 0))
        #self.movex(vx/2 - offset)
        #return
      else:
        self.movex(vx/2 - offset)
        return
    else:
      if not self.colrect_bg_collide(side, avx):
        self.rect_move((vx, 0))
        return
      else:
        self.movex(vx/2 - offset)
        return

      
    vel = vel.normalized() * avx
    self.rect_move((vel.x, vel.y))


  def loop_horizontally(self):
    if self.rect.centerx > SCREENRECT.right:
      self.rect_move((-SCREENRECT.width, 0))
    elif self.rect.centerx < SCREENRECT.left:
      self.rect_move((SCREENRECT.width, 0))

  def rect_move(self, xy):
    self.rect.move_ip(xy)
    self.colrect.move_ip(xy)
    self.visrect.move_ip(xy)

  def rect_place(self, xy):
    self.rect.center = xy
    self.colrect.center = xy
    self.visrect.center = xy


  def update(self):
    self.action()
    self.phy_step()
    self.move()
    self.loop_horizontally()



class TextSprite(pygame.sprite.Sprite):
  
  def __init__(self, size=30):
    pygame.sprite.Sprite.__init__(self)
    self.font = pygame.font.Font(pygame.font.get_default_font(), size)
    self.image = self.font.render('', True, (0,0,0))
    self.rect = self.image.get_rect()

  def set_text(self, text, color=(0,0,0)):
    self.image = self.font.render(text, True, color)
    self.rect = self.image.get_rect()
